﻿using Hotline.FlowEngine.Definitions;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.FlowEngine;
using SqlSugar;
using XF.Domain.Repository;

namespace Hotline.FlowEngine.Workflows;

public abstract class StepBasicEntity : CreationEntity
{
    public string WorkflowId { get; set; }

    #region Difinition

    public string Name { get; set; }

    /// <summary>
    /// 模板内唯一
    /// </summary>
    public string Code { get; set; }

    public EStepType StepType { get; init; }

    /// <summary>
    /// 节点业务类型
    /// </summary>
    public EBusinessType BusinessType { get; set; }

    /// <summary>
    /// 办理人类型
    /// </summary>
    public EHandlerType HandlerType { get; set; }
    
    /// <summary>
    /// 是否有否决按钮
    /// </summary>
    public bool CanReject { get; set; }
    
    #endregion

    // /// <summary>
    // /// 审批者分类（或是直接保存处理者）
    // /// <example>
    // /// 根据类型可能为：roles, depLevels, depTypes, depCodes, userIds
    // /// </example>
    // /// </summary>
    // [SugarColumn(ColumnDataType = "json", IsJson = true)]
    // public List<IdName> HandlerClassifies { get; set; } = new();

    // /// <summary>
    // /// 当前环节会签模式
    // /// </summary>
    // public ECountersignMode CountersignMode { get; set; }

    /// <summary>
    /// 该节点指派办理对象（依据不同指派方式可能为：orgId或userId）,该字段subStep才会存在，stepBox不存在
    /// 采用list类型，兼容多个办理对象可以办理同一个节点的场景
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true)]
    public List<Kv> Handlers { get; set; } = new();

    // /// <summary>
    // /// 办理时间限制(如：24小时、7个工作日)
    // /// </summary>
    // public string? TimeLimit { get; set; }

    // /// <summary>
    // /// 节点属性配置
    // /// </summary>
    // [SugarColumn(ColumnDataType = "json", IsJson = true)]
    // public List<string> Components { get; set; } = new();
    //
    // /// <summary>
    // /// 扩展信息
    // /// </summary>
    // [SugarColumn(ColumnDataType = "json", IsJson = true)]
    // public StepExtension Extension { get; set; } = new();
    //
    // /// <summary>
    // /// 自动路径策略
    // /// </summary>
    // [SugarColumn(DefaultValue = "0")]
    // public EPathPolicy PathPolicy { get; set; }
    
    #region 接办

    /// <summary>
    /// 接办人
    /// </summary>
    public string? AcceptorId { get; set; }

    public string? AcceptorName { get; set; }

    /// <summary>
    /// 接办人部门code
    /// </summary>
    public string? AcceptorOrgId { get; set; }

    public string? AcceptorOrgName { get; set; }
    
    /// <summary>
    /// 接办人部门行政区划代码
    /// </summary>
    public string? AcceptorOrgAreaCode { get; set; }

    /// <summary>
    /// 接办人部门行政区划名称
    /// </summary>
    public string? AcceptorOrgAreaName { get; set; }

    /// <summary>
    /// 接办时间
    /// </summary>
    public DateTime? AcceptTime { get; set; }

    #endregion
    
    #region 办理

    /// <summary>
    /// 办理人
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? HandlerId { get; set; }

    [SugarColumn(IsNullable = true)]
    public string? HandlerName { get; set; }

    /// <summary>
    /// 办理人部门code
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? HandlerOrgId { get; set; }

    /// <summary>
    /// 办理人部门名称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? HandlerOrgName { get; set; }

    /// <summary>
    /// 办理人部门行政区划代码
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? HandlerOrgAreaCode { get; set; }

    /// <summary>
    /// 办理人部门行政区划名称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? HandlerOrgAreaName { get; set; }

    /// <summary>
    /// 办理完成时间
    /// </summary>
    public DateTime? HandleTime { get; set; }

    #endregion
    
    #region 办理参数

    /// <summary>
    /// (下一节点办理人)根据审批者类型不同，此字段为不同内容
    /// <example>
    /// 部门等级/分类为：orgCodes, 角色为：userIds
    /// </example>
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true)]
    public List<Kv> NextHandlers { get; set; } = new();

    /// <summary>
    /// 下一节点主办，(NextHandlers其中一个, 如果不是会签则只有一个)
    /// </summary>
    public string? NextMainHandler { get; set; }

    /// <summary>
    /// 下一节点code（stepBox无值）
    /// </summary>
    public string NextStepCode { get; set; }

    /// <summary>
    /// 是否短信通知
    /// </summary>
    public bool IsSms { get; set; }

    /// <summary>
    /// 办理意见
    /// </summary>
    [SugarColumn(Length = 2000)]
    public string Opinion { get; set; }

    /// <summary>
    /// 附件
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true)]
    public List<string> Additions { get; set; } = new();

    /// <summary>
    /// 当前节点到期时间(期满时间)
    /// </summary>
    public DateTime ExpiredTime { get; set; }

    #endregion

    #region method

    public bool IsCenter() => BusinessType is EBusinessType.Center or EBusinessType.Send;

    public bool IsOrg() => BusinessType is EBusinessType.Department;

    #endregion
}